/*
 *
 *  BlueZ - Bluetooth protocol stack for Linux
 *
 *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
 *
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include <dbus/dbus-glib.h>

#include "client.h"

#include "marshal.h"

#include "dbus-glue.h"

static DBusGConnection *conn;

static GtkTreeStore *store;

#define BLUETOOTH_CLIENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
				BLUETOOTH_TYPE_CLIENT, BluetoothClientPrivate))

typedef struct _BluetoothClientPrivate BluetoothClientPrivate;

struct _BluetoothClientPrivate {
	gboolean registered;
};

G_DEFINE_TYPE(BluetoothClient, bluetooth_client, G_TYPE_OBJECT)

static void bluetooth_client_init(BluetoothClient *self)
{
	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(self);

	priv->registered = FALSE;
}

static void bluetooth_client_finalize(GObject *object)
{
	//BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(object);
}

static void bluetooth_client_set_property(GObject *object, guint prop_id,
					const GValue *value, GParamSpec *pspec)
{
	//BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(object);

	switch (prop_id) {
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}

static void bluetooth_client_get_property(GObject *object, guint prop_id,
					GValue *value, GParamSpec *pspec)
{
	//BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(object);

	switch (prop_id) {
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}

static void mode_changed(DBusGProxy *object,
				const char *mode, gpointer user_data)
{
}

static void timeout_changed(DBusGProxy *object,
				const guint32 timeout, gpointer user_data)
{
}

static void name_changed(DBusGProxy *object,
				const char *name, gpointer user_data)
{
	GtkTreePath *path = user_data;
	GtkTreeIter iter;

	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);

	gtk_tree_store_set(store, &iter, COLUMN_NAME, name, -1);
}

static void minor_changed(DBusGProxy *object,
				const char *minor, gpointer user_data)
{
}

gchar *bluetooth_type_to_string(guint type)
{
	switch (type) {
	case BLUETOOTH_TYPE_PHONE:
		return "Phone";
	case BLUETOOTH_TYPE_MODEM:
		return "Modem";
	case BLUETOOTH_TYPE_COMPUTER:
		return "Computer";
	case BLUETOOTH_TYPE_NETWORK:
		return "Network";
	case BLUETOOTH_TYPE_HEADSET:
		return "Headset";
	case BLUETOOTH_TYPE_KEYBOARD:
		return "Keyboard";
	case BLUETOOTH_TYPE_MOUSE:
		return "Mouse";
	case BLUETOOTH_TYPE_CAMERA:
		return "Camera";
	case BLUETOOTH_TYPE_PRINTER:
		return "Printer";
	default:
		return "Unknown";
	}
}

static guint class_to_type(guint32 class)
{
	switch ((class & 0x1f00) >> 8) {
	case 0x01:
		return BLUETOOTH_TYPE_COMPUTER;
	case 0x02:
		switch ((class & 0xfc) >> 2) {
		case 0x01:
		case 0x02:
		case 0x03:
		case 0x05:
			return BLUETOOTH_TYPE_PHONE;
		case 0x04:
			return BLUETOOTH_TYPE_MODEM;
		}
		break;
	case 0x03:
		return BLUETOOTH_TYPE_NETWORK;
	case 0x04:
		switch ((class & 0xfc) >> 2) {
		case 0x01:
			return BLUETOOTH_TYPE_HEADSET;
		}
		break;
	case 0x05:
		switch ((class & 0xc0) >> 6) {
		case 0x01:
			return BLUETOOTH_TYPE_KEYBOARD;
		case 0x02:
			return BLUETOOTH_TYPE_MOUSE;
		}
		break;
	case 0x06:
		if (class & 0x80)
			return BLUETOOTH_TYPE_PRINTER;
		if (class & 0x20)
			return BLUETOOTH_TYPE_CAMERA;
		break;
	}

	return BLUETOOTH_TYPE_ANY;
}

static void insert_device(DBusGProxy *object, GtkTreeIter *parent,
			const char *address, const int rssi, gboolean active)
{
	GtkTreeIter iter;
	const char *name = NULL;
	guint type;
	guint32 class = 0;
	gboolean trusted = FALSE, connected = FALSE;
	gboolean cont;

	dbus_g_proxy_call(object, "GetRemoteName", NULL,
				G_TYPE_STRING, address, G_TYPE_INVALID,
				G_TYPE_STRING, &name, G_TYPE_INVALID);

	dbus_g_proxy_call(object, "GetRemoteClass", NULL,
				G_TYPE_STRING, address, G_TYPE_INVALID,
				G_TYPE_UINT, &class, G_TYPE_INVALID);

	dbus_g_proxy_call(object, "HasBonding", NULL,
				G_TYPE_STRING, address, G_TYPE_INVALID,
				G_TYPE_BOOLEAN, &trusted, G_TYPE_INVALID);

	dbus_g_proxy_call(object, "IsConnected", NULL,
				G_TYPE_STRING, address, G_TYPE_INVALID,
				G_TYPE_BOOLEAN, &connected, G_TYPE_INVALID);

	cont = gtk_tree_model_iter_children(GTK_TREE_MODEL(store),
							&iter, parent);

	type = class_to_type(class);

	while (cont == TRUE) {
		gchar *value;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_ADDRESS, &value, -1);

		if (g_ascii_strcasecmp(address, value) == 0) {
			gtk_tree_store_set(store, &iter,
						COLUMN_ACTIVE, active,
						COLUMN_CLASS, class,
						COLUMN_RSSI, rssi,
						COLUMN_NAME, name,
						COLUMN_TYPE, type,
						COLUMN_TRUSTED, trusted,
						COLUMN_CONNECTED, connected, -1);
			return;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	gtk_tree_store_insert_with_values(store, &iter, parent, -1,
					COLUMN_ACTIVE, active,
					COLUMN_ADDRESS, address,
					COLUMN_CLASS, class,
					COLUMN_RSSI, rssi,
					COLUMN_NAME, name,
					COLUMN_TYPE, type,
					COLUMN_TRUSTED, trusted,
					COLUMN_CONNECTED, connected, -1);
}

static void invalidate_device(DBusGProxy *object,
				GtkTreeIter *parent, const char *address)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_iter_children(GTK_TREE_MODEL(store),
							&iter, parent);

	while (cont == TRUE) {
		gchar *value;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_ADDRESS, &value, -1);

		if (g_ascii_strcasecmp(address, value) == 0) {
			gtk_tree_store_set(store, &iter,
						COLUMN_ACTIVE, FALSE, -1);
			return;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}
}

static void rename_device(DBusGProxy *object, GtkTreeIter *parent,
					const char *address, const char *name)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_iter_children(GTK_TREE_MODEL(store),
							&iter, parent);

	while (cont == TRUE) {
		gchar *value;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_ADDRESS, &value, -1);

		if (g_ascii_strcasecmp(address, value) == 0) {
			gtk_tree_store_set(store, &iter,
						COLUMN_NAME, name, -1);
			return;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}
}

static void device_found(DBusGProxy *object, const char *address,
		const unsigned int class, const int rssi, gpointer user_data)
{
	GtkTreePath *path = user_data;
	GtkTreeIter iter;

	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);

	insert_device(object, &iter, address, rssi, TRUE);
}

static void device_disappeared(DBusGProxy *object,
				const char *address, gpointer user_data)
{
	GtkTreePath *path = user_data;
	GtkTreeIter iter;

	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);

	invalidate_device(object, &iter, address);
}

static void name_updated(DBusGProxy *object,
		const char *address, const char *name, gpointer user_data)
{
	GtkTreePath *path = user_data;
	GtkTreeIter iter;

	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);

	rename_device(object, &iter, address, name);
}

static void update_adapter(DBusGProxy *object, GtkTreeIter *iter)
{
	GError *error = NULL;
	char **array = NULL;
	char *address = NULL, *mode = NULL;
	char *name = NULL, *major = NULL, *minor = NULL;
	const guint32 timeout;

	adapter_get_address(object, &address, NULL);

	adapter_get_mode(object, &mode, NULL);

	dbus_g_proxy_call(object, "GetDiscoverableTimeout", NULL, G_TYPE_INVALID,
				G_TYPE_UINT, &timeout, G_TYPE_INVALID);

	adapter_get_name(object, &name, NULL);

	adapter_get_major_class(object, &major, NULL);

	adapter_get_minor_class(object, &minor, NULL);

	gtk_tree_store_set(store, iter, COLUMN_ADDRESS, address,
						COLUMN_NAME, name, -1);

	//adapter_list_remote_devices(object, &array, &error);

	adapter_list_bondings(object, &array, &error);

	if (error == NULL) {
		while (*array) {
			insert_device(object, iter, *array, 0, FALSE);
			array++;
		}
	} else
		g_error_free(error);
}

static void add_adapter(const char *path)
{
	DBusGProxy *object;
	GtkTreeIter iter;
	GtkTreePath *tree_path;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		gchar *value;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &value,
						COLUMN_OBJECT, &object, -1);

		if (g_ascii_strcasecmp(path, value) == 0) {
			gtk_tree_store_set(store, &iter,
						COLUMN_ACTIVE, TRUE, -1);

			update_adapter(object, &iter);

			return;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	gtk_tree_store_insert_with_values(store, &iter, NULL, -1,
						COLUMN_PATH, path,
						COLUMN_ACTIVE, TRUE, -1);

	object = dbus_g_proxy_new_for_name(conn, "org.bluez",
						path, "org.bluez.Adapter");

	gtk_tree_store_set(store, &iter, COLUMN_OBJECT, object, -1);

	g_object_unref(object);

	tree_path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);

	dbus_g_proxy_add_signal(object, "ModeChanged",
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "ModeChanged",
				G_CALLBACK(mode_changed), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "DiscoverableTimeoutChanged",
					G_TYPE_UINT, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "DiscoverableTimeoutChanged",
				G_CALLBACK(timeout_changed), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "NameChanged",
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "NameChanged",
				G_CALLBACK(name_changed), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "MinorClassChanged",
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "MinorClassChanged",
				G_CALLBACK(minor_changed), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "RemoteDeviceFound",
						G_TYPE_STRING, G_TYPE_UINT,
						G_TYPE_INT, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "RemoteDeviceFound",
				G_CALLBACK(device_found), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "RemoteDeviceDisappeared",
						G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "RemoteDeviceDisappeared",
				G_CALLBACK(device_disappeared), tree_path, NULL);

	dbus_g_proxy_add_signal(object, "RemoteNameUpdated",
				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "RemoteNameUpdated",
				G_CALLBACK(name_updated), tree_path, NULL);

	update_adapter(object, &iter);
}

static void remove_adapter(const char *path)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		gchar *value;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &value, -1);

		if (g_ascii_strcasecmp(path, value) == 0) {
			gtk_tree_store_set(store, &iter,
						COLUMN_ACTIVE, FALSE,
						COLUMN_ADDRESS, NULL,
						COLUMN_NAME, NULL, -1);
			return;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}
}

static void adapter_added(DBusGProxy *object,
				const char *path, gpointer user_data)
{
	add_adapter(path);
}

static void adapter_removed(DBusGProxy *object,
				const char *path, gpointer user_data)
{
	remove_adapter(path);
}

static void setup_manager(void)
{
	DBusGProxy *object;
	GError *error = NULL;
	char *default_adapter = NULL;
	char **array = NULL;

	object = dbus_g_proxy_new_for_name(conn, "org.bluez",
					"/org/bluez", "org.bluez.Manager");

	dbus_g_proxy_add_signal(object, "AdapterAdded",
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "AdapterAdded",
				G_CALLBACK(adapter_added), conn, NULL);

	dbus_g_proxy_add_signal(object, "AdapterRemoved",
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "AdapterRemoved",
				G_CALLBACK(adapter_removed), conn, NULL);

	manager_default_adapter(object, &default_adapter, NULL);

	manager_list_adapters(object, &array, &error);

	if (error == NULL) {
		while (*array) {
			add_adapter(*array);
			array++;
		}
	} else
		g_error_free(error);
}

static void name_owner_changed(DBusGProxy *object, const char *name,
		const char *prev, const char *new, gpointer user_data)
{
	if (!g_ascii_strcasecmp(name, "org.bluez") && *new == '\0') {
		/* disable adapters */
	}
}

static void setup_dbus(void)
{
	DBusGProxy *object;
	GError *error = NULL;

	dbus_g_object_register_marshaller(marshal_VOID__STRING_UINT_INT,
				G_TYPE_NONE, G_TYPE_STRING, G_TYPE_UINT,
						G_TYPE_INT, G_TYPE_INVALID);

	dbus_g_object_register_marshaller(marshal_VOID__STRING_STRING,
						G_TYPE_NONE, G_TYPE_STRING,
						G_TYPE_STRING, G_TYPE_INVALID);

	conn = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
	if (error != NULL) {
		g_printerr("Connecting to system bus failed: %s\n",
							error->message);
		g_error_free(error);
		return;
	}

	object = dbus_g_proxy_new_for_name(conn, DBUS_SERVICE_DBUS,
					DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS);

	dbus_g_proxy_add_signal(object, "NameOwnerChanged",
					G_TYPE_STRING, G_TYPE_STRING,
					G_TYPE_STRING, G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(object, "NameOwnerChanged",
				G_CALLBACK(name_owner_changed), NULL, NULL);
}

static void bluetooth_client_class_init(BluetoothClientClass *klass)
{
	g_type_class_add_private(klass, sizeof(BluetoothClientPrivate));

	G_OBJECT_CLASS(klass)->finalize = bluetooth_client_finalize;

	G_OBJECT_CLASS(klass)->set_property = bluetooth_client_set_property;
	G_OBJECT_CLASS(klass)->get_property = bluetooth_client_get_property;

	store = gtk_tree_store_new(10, G_TYPE_STRING, G_TYPE_BOOLEAN,
			G_TYPE_OBJECT, G_TYPE_STRING, G_TYPE_UINT,
			G_TYPE_INT, G_TYPE_STRING, G_TYPE_UINT,
			G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);

	setup_dbus();

	setup_manager();
}

BluetoothClient *bluetooth_client_new(void)
{
	return BLUETOOTH_CLIENT(g_object_new(BLUETOOTH_TYPE_CLIENT, NULL));
}

void bluetooth_client_destroy(BluetoothClient *self)
{
	g_object_unref(self);
}

gboolean bluetooth_client_register_passkey_agent(BluetoothClient *self,
		const char *path, const char *address, const void *info)
{
	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(self);
	DBusGProxy *proxy;

	if (priv->registered == FALSE) {
		dbus_g_object_type_install_info(BLUETOOTH_TYPE_CLIENT, info);

		dbus_g_connection_register_g_object(conn, path, G_OBJECT(self));

		priv->registered = TRUE;
	}

	proxy = dbus_g_proxy_new_for_name(conn, "org.bluez",
				"/org/bluez/hci0", "org.bluez.Security");

	security_register_passkey_agent(proxy, path, address, NULL);

	g_object_unref(proxy);

	return TRUE;
}

static void create_bonding_reply(DBusGProxy *proxy, GError *error, gpointer userdata)
{
	//g_printf("create bonding reply\n");
}

gboolean bluetooth_client_create_bonding(BluetoothClient *self,
					gchar *adapter, const gchar *address)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		DBusGProxy *object;
		gchar *path;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &path,
						COLUMN_OBJECT, &object, -1);

		if (g_ascii_strcasecmp(path, adapter) == 0) {
			adapter_create_bonding_async(object, address,
						create_bonding_reply, NULL);
			return TRUE;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	return FALSE;
}

gboolean bluetooth_client_discover_devices(BluetoothClient *self, gchar *adapter)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		DBusGProxy *object;
		gchar *path;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &path,
						COLUMN_OBJECT, &object, -1);

		if (g_ascii_strcasecmp(path, adapter) == 0) {
			dbus_g_proxy_call(object, "DiscoverDevices",
							NULL, G_TYPE_INVALID);
			return TRUE;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	return FALSE;
}

gboolean bluetooth_client_cancel_discovery(BluetoothClient *self, gchar *adapter)
{
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		DBusGProxy *object;
		gchar *path;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &path,
						COLUMN_OBJECT, &object, -1);

		if (g_ascii_strcasecmp(path, adapter) == 0) {
			dbus_g_proxy_call(object, "CancelDiscovery",
							NULL, G_TYPE_INVALID);
			return TRUE;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	return FALSE;
}

GtkTreeModel *bluetooth_client_get_model(BluetoothClient *self)
{
	g_object_ref(store);

	return GTK_TREE_MODEL(store);
}

GtkTreeModel *bluetooth_client_get_model_simple(BluetoothClient *self)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean cont;

	model = gtk_tree_model_filter_new(GTK_TREE_MODEL(store), NULL);

	gtk_tree_model_filter_set_visible_column(GTK_TREE_MODEL_FILTER(model),
								COLUMN_ACTIVE);

	if (gtk_tree_model_iter_n_children(model, NULL) != 1)
		return model;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		gboolean active;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_ACTIVE, &active, -1);

		if (active == TRUE) {
			GtkTreePath *path;

			path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);

			g_object_unref(model);

			model = gtk_tree_model_filter_new(GTK_TREE_MODEL(store), path);

			gtk_tree_path_free(path);

			break;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	return model;
}

static gboolean adapter_list_filter(GtkTreeModel *model,
					GtkTreeIter *iter, gpointer data)
{
	gchar *path;

	gtk_tree_model_get(model, iter, COLUMN_PATH, &path, -1);

	if (path) {
		g_free(path);
		return TRUE;
	} else
		return FALSE;
}

GtkTreeModel *bluetooth_client_get_model_adapter_list(BluetoothClient *self)
{
	GtkTreeModel *model;

	model = gtk_tree_model_filter_new(GTK_TREE_MODEL(store), NULL);

	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(model),
					adapter_list_filter, NULL, NULL);

	return model;
}

static gboolean device_active_filter(GtkTreeModel *model,
					GtkTreeIter *iter, gpointer data)
{
	gboolean active;

	gtk_tree_model_get(model, iter, COLUMN_ACTIVE, &active, -1);

	return active;
}

GtkTreeModel *bluetooth_client_get_model_for_adapter(BluetoothClient *self,
								gchar *adapter)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean cont;

	cont = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	while (cont == TRUE) {
		gchar *path;

		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
						COLUMN_PATH, &path, -1);

		if (!g_ascii_strcasecmp(path, adapter)) {
			GtkTreePath *path;

			path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);

			model = gtk_tree_model_filter_new(GTK_TREE_MODEL(store), path);

			gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(model),
							device_active_filter, NULL, NULL);

			gtk_tree_path_free(path);

			return model;
		}

		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}

	return NULL;
}
